// For bme-280
//----------------------------------------------------------------
#define UART_ID uart0

#define LED1 2
#define LED2 27

#define BME_ID 0x76  // BMP280 device ID

#define hum_lsb 0xFE;       // humidity byte 0      Read Only  bmpdata[7]
#define hum_msb 0xFD;       // humidity byte 1      Read Only  bmpdata[6]
#define temp_xlsb 0xFC;     // temperature byte 0   Read Only  bmpdata[5]   R,Address,Data
#define temp_lsb 0xFB;      // temperature byte 1   Read Only  bmpdata[4]
#define temp_msb 0xFA;      // temperature byte 2   Read Only  bmpdata[3]
#define press_xlsb 0xF9;    // pressure byte 0      Read Only  bmpdata[2]
#define press_lsb 0xF8;     // pressure byte 1      Read Only  bmpdata[1]
#define press_msb 0xF7;     // pressure byte 2      Read Only  bmpdata[0]

#define bme_config 0xF5;    // configuration        Read/Write  R/W,Address;Data
#define bme_ctrl_meas 0xF4; // control measurement  Read/Write  R/W,Address;Data
#define bme_status 0xF3;    // status               Read Only
#define bme_reset 0xE0;     // reset                Write Only  /W,Address;Data

#define bme_id_reg 0xD0;    // part id = 0x58       Read Only

#define PB1 14
#define PB2 13
#define PB3 12
#define SW 15

unsigned char i2c_outbuffer[10];
unsigned char i2c_inbuffer[128];
void log_raw(void);

void bme_init(void);
void do_bme280(void);
void show_wx_data_lcd();
void log_wx_data_ts(void);

unsigned char i2c_read_byte(unsigned char);
void i2c_read(unsigned char, unsigned int);
void i2c_write_byte_bme(unsigned char, unsigned char);
void i2c_write(unsigned char, unsigned int);
int wait_ready(void);
int do_compensation(void);

void log_time(void);   // time stamp in 'line' string
void i2c_get_time(void);

void line_fill(unsigned char, char *, unsigned char, unsigned char);

unsigned char bmedata[8];
unsigned char caldata[24];      // bme calibration data dig T1 -T3, dig P1 - P9, dighum
unsigned char humdata[8];

uint16_t dig_T1;
int16_t dig_T2;
int16_t dig_T3;

uint16_t dig_P1;
int16_t dig_P2;
int16_t dig_P3;
int16_t dig_P4;
int16_t dig_P5;
int16_t dig_P6;
int16_t dig_P7;
int16_t dig_P8;
int16_t dig_P9;

uint8_t dig_H1;
int16_t dig_H2;
uint8_t dig_H3;
int16_t dig_H4;
int16_t dig_H5;
int8_t dig_H6;

int32_t t_fine;
int32_t adc_t, adc_temp, adc_p;
uint32_t adc_h;

int32_t T, Fdeg, hextemp;
uint32_t Humidity, Humidity3, Pressure;

char nstr[128];   // new string segment
char line[512];   // string for uart
extern unsigned char device_id;

void itoa_hex(unsigned char);
void itoa_hex_long(unsigned long);
void itoa_hex_int(unsigned int);
void lcd_byte(unsigned char);    // clear screen
void read_bme280(void);
void getcal(void);
void log_cal(void);

extern void lcd_wr_data(unsigned char);
extern void do_led_break(void);
extern void SetCmdMode(void);
extern void lcd_wr_instr(unsigned char);

unsigned char temperature_lcd[10];
unsigned char humidity_lcd[4];
unsigned char pressure_lcd[10];
unsigned char dev[6];

void pressure_ltoa(char *);
void temperature_itoa(char *);
void humidity_btoa(char *);
void devid(char *str);

unsigned char temperature_bcd[5];
unsigned char humidity_bcd[2];
unsigned char pressure_bcd[6];

signed int temperature;
unsigned long pressure;
unsigned char humidity;

void log_digs(void);
unsigned char test_buffer[32];
signed long testvars[32];
void log_vars(void);
void lcd_display(void);

unsigned long show_temperature, show_humidity, show_pressure;
//=========================================
uint32_t compensate_pressure(void);
static int32_t compensate_temperature(void);
static uint32_t compensate_humidity(void);

// For reading the ds3231 clock
#define SECS   0
#define MINS   1
#define HOURS  2
#define DAY    3
#define DATE   4
#define MONTH  5
#define YEAR   6
extern unsigned char time_buffer[];

void log_time(void);
void my_itoa(unsigned int,char *);

extern int secs_flag;
void LED_break(void);
extern unsigned char uart_char;

extern unsigned char temp_buffer[256];
extern struct log flash_entry;   // pump log in RAM, 32 log entries
extern const char version[64];
extern uint16_t Page_Index, Last_Page, Sector_Index;
extern void do_start_date(uint16_t);
extern char start_date[20];
extern unsigned int ts_number;
extern unsigned int Entry_Read_Index, Flash_Write_Index, erase_flag, reset_rx_start;
void do_uart_RX(void);
extern void make_entry(void);
extern void test_read_sector(uint16_t);
extern void myflash_erase_sector(uint16_t);
extern void erase_sector(uint16_t);
extern void erase_all_sectors(void);
extern void dump_flash(uint32_t, uint32_t);
extern void write_page(uint32_t, unsigned char *);
extern void test10(void);
extern void show_log(struct log *);
void show_entry(struct log *);
extern const char day_array[7][3];
void my_hextoa2(unsigned char, char *);

//Time-stamped log 32 bytes, duplicated in myflash.h 
struct log
{
   uint16_t ts_number;     // index number                   0,1
   
   uint8_t ts_date;       // ASCII time-stamped date        2
   uint8_t ts_day;        // ASCII time-stamped day         3
   uint8_t ts_month;      // ASCII time-stamped month       4
   uint8_t ts_year;       // ASCII time-stamped year        5

   uint8_t ts_hours;      // ASCII time-stamped hour        6
   uint8_t ts_minutes;    // ASCII time-stamped minutes     7
   uint8_t ts_seconds;    // ASCII time-stamped second      8

   int32_t Ftemperature;  //  Hes                           9, 10, 11, 12
   uint32_t Pressure;     //  Hex                           13, 14 15, 16  
   uint32_t Humidity;     //  Hex                           17, 18, 19, 20

   uint8_t data[10];      // just Hex data                  21 - 30
                      
   uint8_t flag;          // flag for non-blank             31
};

void log_wx_data_entry(struct log *);
void temperature_itoa_str(int32_t, char *);      // remove leading zeros; end with NULL
void pressure_ltoa_str(uint32_t, char *);        // remove leading zeros; end with NULL
void humidity_btoa_str(uint8_t, char *);
void make_wx_entry(struct log *);
extern void show_log_page(uint32_t);

uint16_t bme_log_count, bme_log_flag;

//==================
static double dcompensate_humidity(void);
static double dcompensate_pressure(void);
static double dcompensate_temperature(void);
void sint32_bcd_str(int32_t, char *);    // int32_t to BCD string
void uint32_bcd_str(uint32_t, char *);   //uint32_t to BCD string

static void test_bosch_parse_calib_data(void);
